"""Open (closed) Index Singleton"""
import click
from curator.cli_singletons.object_class import cli_action
from curator.cli_singletons.utils import get_width, validate_filter_json

@click.command(name='open', context_settings=get_width())
@click.option(
    '--ignore_empty_list',
    is_flag=True,
    help='Do not raise exception if there are no actionable indices'
)
@click.option(
    '--allow_ilm_indices/--no-allow_ilm_indices',
    help='Allow Curator to operate on Index Lifecycle Management monitored indices.',
    default=False,
    show_default=True
)
@click.option(
    '--filter_list',
    callback=validate_filter_json,
    help='JSON array of filters selecting indices to act on.',
    required=True
)
@click.pass_context
def open_indices(ctx, ignore_empty_list, allow_ilm_indices, filter_list):
    """
    Open Indices
    """
    # ctx.info_name is the name of the function or name specified in @click.command decorator
    action = cli_action(
        ctx.info_name,
        ctx.obj['config']['client'],
        {'allow_ilm_indices':allow_ilm_indices},
        filter_list,
        ignore_empty_list
    )
    action.do_singleton_action(dry_run=ctx.obj['dry_run'])



import opensearchpy
import curator
import os
import json
import string
import random
import tempfile
from time import sleep
import click
from click import testing as clicktest
from mock import patch, Mock

from . import CuratorTestCase
from . import testvars as testvars

import logging
logger = logging.getLogger(__name__)

host, port = os.environ.get('TEST_ES_SERVER', 'localhost:9200').split(':')
port = int(port) if port else 9200

class TestActionFileforceMerge(CuratorTestCase):
    def test_merge(self):
        count = 1
        idx = 'my_index'
        self.create_index(idx)
        self.add_docs(idx)
        ilo1 = curator.IndexList(self.client)
        ilo1._get_segment_counts()
        self.assertEqual(3, ilo1.index_info[idx]['segments'])
        self.write_config(
            self.args['configfile'], testvars.client_config.format(host, port))
        self.write_config(self.args['actionfile'],
            testvars.forcemerge_test.format(count, 0.9))
        test = clicktest.CliRunner()
        _ = test.invoke(
            curator.cli,
            ['--config', self.args['configfile'], self.args['actionfile']],
        )
        ilo2 = curator.IndexList(self.client)
        # This stupid block is only for the benefit of Travis CI
        # With Python 2.7 and ES 7.0, it apparently can finish testing before
        # the segments have _reported_ as fully merged. This is forcing
        # 3 checks before giving up and reporting the result.
        for _ in range(0, 3):   
            self.client.indices.refresh(index=idx)
            ilo2._get_segment_counts()
            if ilo2.index_info[idx]['segments'] == count:
                break
            else:
                sleep(1)
        self.assertEqual(count, ilo2.index_info[idx]['segments'])
    def test_extra_option(self):
        self.write_config(
            self.args['configfile'], testvars.client_config.format(host, port))
        self.write_config(self.args['actionfile'],
            testvars.bad_option_proto_test.format('forcemerge'))
        test = clicktest.CliRunner()
        result = test.invoke(
                    curator.cli,
                    [
                        '--config', self.args['configfile'],
                        self.args['actionfile']
                    ],
                    )
        self.assertEqual(1, result.exit_code)

class TestCLIforceMerge(CuratorTestCase):
    def test_merge(self):
        count = 1
        idx = 'my_index'
        self.create_index(idx)
        self.add_docs(idx)
        ilo1 = curator.IndexList(self.client)
        ilo1._get_segment_counts()
        self.assertEqual(3, ilo1.index_info[idx]['segments'])
        args = self.get_runner_args()
        args += [
            '--config', self.args['configfile'],
            'forcemerge',
            '--max_num_segments', str(count),
            '--delay', '0.9',
            '--filter_list', '{"filtertype":"pattern","kind":"prefix","value":"my"}',
        ]
        self.assertEqual(0, self.run_subprocess(args, logname='TestCLIforceMerge.test_merge'))
        ilo2 = curator.IndexList(self.client)
        # This stupid block is only for the benefit of Travis CI
        # With Python 2.7 and ES 7.0, it apparently can finish testing before
        # the segments have _reported_ as fully merged. This is forcing
        # 3 checks before giving up and reporting the result.
        for _ in range(0, 3):   
            self.client.indices.refresh(index=idx)
            ilo2._get_segment_counts()
            if ilo2.index_info[idx]['segments'] == count:
                break
            else:
                sleep(1)
        self.assertEqual(count, ilo2.index_info[idx]['segments'])



"""Configuration utilty functions"""
import logging
from copy import deepcopy
from voluptuous import Schema
from curator.validators import SchemaCheck, config_file
from curator.utils import ensure_list, get_yaml, prune_nones, test_client_options
from curator.logtools import LogInfo, Whitelist, Blacklist

def test_config(config):
    """Test YAML against the schema"""
    # Get config from yaml file
    yaml_config = get_yaml(config)
    # if the file is empty, which is still valid yaml, set as an empty dict
    yaml_config = {} if not yaml_config else prune_nones(yaml_config)
    # Voluptuous can't verify the schema of a dict if it doesn't have keys,
    # so make sure the keys are at least there and are dict()
    for k in ['client', 'logging']:
        if k not in yaml_config:
            yaml_config[k] = {}
        else:
            yaml_config[k] = prune_nones(yaml_config[k])
    return SchemaCheck(
        yaml_config, config_file.client(), 'Client Configuration', 'full configuration dictionary'
    ).result()

def set_logging(log_opts):
    """Configure global logging options"""
    # Set up logging
    loginfo = LogInfo(log_opts)
    logging.root.addHandler(loginfo.handler)
    logging.root.setLevel(loginfo.numeric_log_level)
    _ = logging.getLogger('curator.cli')
    # Set up NullHandler() to handle nested opensearchpy.trace Logger
    # instance in opensearch-py python client
    logging.getLogger('opensearchpy.trace').addHandler(logging.NullHandler())
    if log_opts['blacklist']:
        for bl_entry in ensure_list(log_opts['blacklist']):
            for handler in logging.root.handlers:
                handler.addFilter(Blacklist(bl_entry))

def process_config(yaml_file):
    """Process yaml_file and return a valid client configuration"""
    config = test_config(yaml_file)
    set_logging(config['logging'])
    test_client_options(config['client'])
    return config['client']

def password_filter(data):
    """
    Return a deepcopy of the dictionary with any password fields hidden
    """
    def iterdict(mydict):
        for key, value in mydict.items():
            if isinstance(value, dict):
                iterdict(value)
            elif key == "http_auth":
                mydict.update({"http_auth": (value[0],"REDACTED")})
            elif key == "password":
                mydict.update({"password": "REDACTED"})
        return mydict
    return iterdict(deepcopy(data))



"""Allocation Singleton"""
import click
from curator.cli_singletons.object_class import cli_action
from curator.cli_singletons.utils import get_width, validate_filter_json

@click.command(context_settings=get_width())
@click.option('--key', type=str, required=True, help='Node identification tag')
@click.option('--value', type=str, default=None, help='Value associated with --key')
@click.option('--allocation_type', type=click.Choice(['require', 'include', 'exclude']))
@click.option(
    '--wait_for_completion/--no-wait_for_completion',
    default=False,
    help='Wait for the allocation to complete',
    show_default=True
)
@click.option(
    '--max_wait',
    default=-1,
    type=int,
    help='Maximum number of seconds to wait_for_completion',
    show_default=True
)
@click.option(
    '--wait_interval',
    default=9,
    type=int,
    help='Seconds to wait between completion checks.',
    show_default=True
)
@click.option(
    '--ignore_empty_list',
    is_flag=True,
    help='Do not raise exception if there are no actionable indices'
)
@click.option(
    '--allow_ilm_indices/--no-allow_ilm_indices',
    help='Allow Curator to operate on Index Lifecycle Management monitored indices.',
    default=False,
    show_default=True
)
@click.option(
    '--filter_list',
    callback=validate_filter_json,
    help='JSON array of filters selecting indices to act on.',
    required=True
)
@click.pass_context
def allocation(
        ctx,
        key,
        value,
        allocation_type,
        wait_for_completion,
        max_wait,
        wait_interval,
        ignore_empty_list,
        allow_ilm_indices,
        filter_list
    ):
    """
    Shard Routing Allocation
    """
    manual_options = {
        'key': key,
        'value': value,
        'allocation_type': allocation_type,
        'wait_for_completion': wait_for_completion,
        'max_wait': max_wait,
        'wait_interval': wait_interval,
        'allow_ilm_indices': allow_ilm_indices,
    }
    # ctx.info_name is the name of the function or name specified in @click.command decorator
    action = cli_action(
        ctx.info_name, ctx.obj['config']['client'], manual_options, filter_list, ignore_empty_list)
    action.do_singleton_action(dry_run=ctx.obj['dry_run'])



import os
import re
import sys
from setuptools import setup

# Utility function to read from file.
def fread(fname):
    return open(os.path.join(os.path.dirname(__file__), fname)).read()

def get_version():
    VERSIONFILE="curator/_version.py"
    verstrline = fread(VERSIONFILE).strip()
    vsre = r"^__version__ = ['\"]([^'\"]*)['\"]"
    mo = re.search(vsre, verstrline, re.M)
    if mo:
        VERSION = mo.group(1)
    else:
        raise RuntimeError("Unable to find version string in %s." % (VERSIONFILE,))
    build_number = os.environ.get('CURATOR_BUILD_NUMBER', None)
    if build_number:
        return VERSION + "b{}".format(build_number)
    return VERSION

def get_install_requires():
    res = ['opensearch-py>=1.0.0' ]
    res.append('urllib3>=1.26.5,<2')
    res.append('requests>=2.26.0')
    res.append('boto3>=1.18.18')
    res.append('requests_aws4auth>=1.1.1')
    res.append('click>=7.0,<8.0')
    res.append('pyyaml>=5.4.1')
    res.append('voluptuous>=0.12.1')
    res.append('certifi>=2021.5.30')
    res.append('six>=1.16.0')
    return res

try:
    ### cx_Freeze ###
    from cx_Freeze import setup, Executable
    try:
        import certifi
        cert_file = certifi.where()
    except ImportError:
        cert_file = ''
    # Dependencies are automatically detected, but it might need
    # fine tuning.


    base = 'Console'

    icon = None
    if os.path.exists('Elastic.ico'):
        icon = 'Elastic.ico'

    curator_exe = Executable(
        "run_curator.py",
        base=base,
        targetName = "curator",
    )
    curator_cli_exe = Executable(
        "run_singleton.py",
        base=base,
        targetName = "curator_cli",
    )
    repomgr_exe = Executable(
        "run_es_repo_mgr.py",
        base=base,
        targetName = "es_repo_mgr",
    )
    build_dict = { 
        "build_exe": dict(
            packages = [],
            excludes = [],
            include_files = [cert_file],
        )
    }
    if sys.platform == "win32":
        curator_exe = Executable(
            "run_curator.py",
            base=base,
            targetName = "curator.exe",
            icon = icon
        )
        curator_cli_exe = Executable(
            "run_singleton.py",
            base=base,
            targetName = "curator_cli.exe",
            icon = icon
        )
        repomgr_exe = Executable(
            "run_es_repo_mgr.py",
            base=base,
            targetName = "es_repo_mgr.exe",
            icon = icon
        )

        msvcrt = 'vcruntime140.dll'
        build_dict = { 
            "build_exe": {
                "include_files": [cert_file, msvcrt],
                "include_msvcr": True, 
                "silent": True,
            },
            "bdist_msi": {
                "upgrade_code": fread("msi_guid.txt"),
                "all_users": True,
                "add_to_path": True,
                "summary_data": {"author": "Elastic", "comments": "version {0}".format(get_version())},
                "install_icon": icon,
            }
        }

    setup(
        name = "curator-opensearch",
        version = get_version(),
        author = "Uzhinsky",
        author_email = "lspci@mail.ru",
        description = "Tending your Elasticsearch indices",
        long_description=fread('README.rst'),
        url = "https://github.com/uzhinskiy/curator-opensearch",
        download_url = "https://github.com/uzhinskiy/curator-opensearch/releases/tag/" + get_version(),
        license = "Apache License, Version 2.0",
        install_requires = get_install_requires(),
        setup_requires = get_install_requires(),
        keywords = "elasticsearch time-series indexed index-expiry",
        packages = ["curator"],
        include_package_data=True,
        entry_points = {
            "console_scripts" : [
                "curator = curator.cli:cli",
                "curator_cli = curator.curator_cli:main",
                "es_repo_mgr = curator.repomgrcli:repo_mgr_cli",
            ]
        },
        classifiers=[
            "Intended Audience :: Developers",
            "Intended Audience :: System Administrators",
            "License :: OSI Approved :: Apache Software License",
            "Operating System :: OS Independent",
            "Programming Language :: Python",
            "Programming Language :: Python :: 3.7",
            "Programming Language :: Python :: 3.8",
            "Programming Language :: Python :: 3.9",
        ],
        test_suite = "test.run_tests.run_all",
        tests_require = ["mock", "nose", "coverage", "nosexcover"],
        options = build_dict,
        executables = [curator_exe, curator_cli_exe, repomgr_exe]
    )
    ### end cx_Freeze ###
except ImportError:
    setup(
        name = "curator-opensearch",
        version = get_version(),
        author = "Uzhinsky",
        author_email = "lspci@mail.ru",
        description = "Tending your Elasticsearch indices",
        long_description=fread('README.rst'),
        url = "https://github.com/uzhinskiy/curator-opensearch",
        download_url = "https://github.com/uzhinskiy/curator-opensearch/releases/tag/" + get_version(),
        license = "Apache License, Version 2.0",
        install_requires = get_install_requires(),
        keywords = "elasticsearch time-series indexed index-expiry",
        packages = ["curator"],
        include_package_data=True,
        entry_points = {
            "console_scripts" : [
                "curator = curator.cli:cli",
                "curator_cli = curator.curator_cli:main",
                "es_repo_mgr = curator.repomgrcli:repo_mgr_cli",
            ]
        },
        classifiers=[
            "Intended Audience :: Developers",
            "Intended Audience :: System Administrators",
            "License :: OSI Approved :: Apache Software License",
            "Operating System :: OS Independent",
            "Programming Language :: Python",
            "Programming Language :: Python :: 3.7",
            "Programming Language :: Python :: 3.8",
            "Programming Language :: Python :: 3.9",
        ],
        test_suite = "test.run_tests.run_all",
        tests_require = ["mock", "nose", "coverage", "nosexcover"]
    )



import opensearchpy
import curator
import os
import click
from click import testing as clicktest

from . import CuratorTestCase
from . import testvars as testvars

import logging
logger = logging.getLogger(__name__)

host, port = os.environ.get('TEST_ES_SERVER', 'localhost:9200').split(':')
port = int(port) if port else 9200

class TestActionFileFreeze(CuratorTestCase):
    def test_freeze_unfrozen(self):
        self.write_config(
            self.args['configfile'], testvars.client_config.format(host, port))
        self.write_config(self.args['actionfile'],
            testvars.optionless_proto.format('freeze'))

        version = curator.utils.get_version(self.client)
        if version <= (6, 6, 0):
            expected = 1
            # Freeze / Unfreeze not supported before 6.6
            self.assertEqual(expected, 1)
            return
        
        self.create_index('my_index')
        self.create_index('dummy')
        cluster_state = self.client.cluster.state(
                metric='metadata',
        )
        self.assertEqual(
            "false",
            cluster_state['metadata']['indices']['my_index']['settings']['index'].get('frozen', "false")
        )
        
        self.assertEqual(
            "false",
            cluster_state['metadata']['indices']['dummy']['settings']['index'].get('frozen', "false")
        )
                
        test = clicktest.CliRunner()
        _ = test.invoke(
                curator.cli,
                [
                    '--config', self.args['configfile'],
                    self.args['actionfile']
                ],
                )
        cluster_state = self.client.cluster.state(
            metric='metadata',
        )
        self.assertEqual(
            "true",
            cluster_state['metadata']['indices']['my_index']['settings']['index'].get('frozen', "false")
        )
        
        self.assertEqual(
            "false",
            cluster_state['metadata']['indices']['dummy']['settings']['index'].get('frozen', "false")
        )
        
    def test_freeze_frozen(self):
        self.write_config(
            self.args['configfile'], testvars.client_config.format(host, port))
        self.write_config(self.args['actionfile'],
            testvars.optionless_proto.format('freeze'))

        version = curator.utils.get_version(self.client)
        if version <= (6, 6, 0):
            expected = 1
            # Freeze / Unfreeze not supported before 6.6
            self.assertEqual(expected, 1)
            return

        self.create_index('my_index')
        # pylint: disable=E1123
        self.client.xpack.indices.freeze(
            index='my_index')
        self.create_index('dummy')
        cluster_state = self.client.cluster.state(
                metric='metadata',
            )
        self.assertEqual(
            "true",
            cluster_state['metadata']['indices']['my_index']['settings']['index'].get('frozen', "false")
        )
        
        self.assertEqual(
            "false",
            cluster_state['metadata']['indices']['dummy']['settings']['index'].get('frozen', "false")
        )        
        test = clicktest.CliRunner()
        _ = test.invoke(
                    curator.cli,
                    [
                        '--config', self.args['configfile'],
                        self.args['actionfile']
                    ],
                    )
        cluster_state = self.client.cluster.state(
                metric='metadata',
            )
        self.assertEqual(
            "true",
            cluster_state['metadata']['indices']['my_index']['settings']['index'].get('frozen', "false")
        )
        
        self.assertEqual(
            "false",
            cluster_state['metadata']['indices']['dummy']['settings']['index'].get('frozen', "false")
        )

    def test_extra_option(self):
        self.write_config(
            self.args['configfile'], testvars.client_config.format(host, port))
        self.write_config(self.args['actionfile'],
            testvars.bad_option_proto_test.format('freeze'))
        self.create_index('my_index')
        self.create_index('dummy')

        version = curator.utils.get_version(self.client)
        if version <= (6, 6, 0):
            expected = 1
            # Freeze / Unfreeze not supported before 6.6
            self.assertEqual(expected, 1)
            return

        cluster_state = self.client.cluster.state(
                metric='metadata',
            )
        self.assertEqual(
            "false",
            cluster_state['metadata']['indices']['my_index']['settings']['index'].get('frozen', "false")
        )
        
        self.assertEqual(
            "false",
            cluster_state['metadata']['indices']['dummy']['settings']['index'].get('frozen', "false")
        )        
        test = clicktest.CliRunner()
        result = test.invoke(
                    curator.cli,
                    [
                        '--config', self.args['configfile'],
                        self.args['actionfile']
                    ],
                    )
        
        cluster_state = self.client.cluster.state(
                metric='metadata',
            )
        self.assertEqual(
            "false",
            cluster_state['metadata']['indices']['my_index']['settings']['index'].get('frozen', "false")
        )
        
        self.assertEqual(
            "false",
            cluster_state['metadata']['indices']['dummy']['settings']['index'].get('frozen', "false")
        )

class TestCLIFreeze(CuratorTestCase):
    def test_freeze_unfrozen(self):
        index = 'my_index'

        version = curator.utils.get_version(self.client)
        if version <= (6, 6, 0):
            expected = 1
            # Freeze / Unfreeze not supported before 6.6
            self.assertEqual(expected, 1)
            return

        self.create_index(index)
        self.create_index('dummy')
        self.create_index('my_other')
        cluster_state = self.client.cluster.state(
                metric='metadata',
            )
        self.assertEqual(
            "false",
            cluster_state['metadata']['indices']['my_index']['settings']['index'].get('frozen', "false")
        )

        self.assertEqual(
            "false",
            cluster_state['metadata']['indices']['my_other']['settings']['index'].get('frozen', "false")
        )
        
        # Now freeze `index` (dummy stays unfrozen)
        
        args = self.get_runner_args()
        args += [
            '--config', self.args['configfile'],
            'freeze',
            '--filter_list', '{"filtertype":"pattern","kind":"prefix","value":"my"}',
        ]
        self.assertEqual(0, self.run_subprocess(args, logname='TestCLIFreeze.test_freeze_unfrozen'))
        cluster_state = self.client.cluster.state(
                metric='metadata',
            )
        self.assertEqual(
            "true",
            cluster_state['metadata']['indices']['my_index']['settings']['index'].get('frozen', "false")
        )

        self.assertEqual(
            "true",
            cluster_state['metadata']['indices']['my_other']['settings']['index'].get('frozen', "false")
        )
        
        self.assertEqual(
            "false",
            cluster_state['metadata']['indices']['dummy']['settings']['index'].get('frozen', "false")
        )

